#include "mbed.h"
#include "USBHostKeyboard.h"

#define LOG_DEBUG
#ifdef LOG_DEBUG
#define log_debug(format,...)   std::printf(format,##__VA_ARGS__)
#else
#define log_debug(format,...)
#endif

typedef enum {
    Init,
    Wait,
    Prenote,
    Delete,
} num_state_t;

typedef enum {
    EXISTING,
    NONE,

    DONE,
    CANTWRITE,
    UNKNOWN
} file_write_t;

typedef enum {
    Number =0,
    Show =!Number
} mode_key_t;
const char *const mode_string[] = {"Number","Show"};

LocalFileSystem local("local");
BusOut led(LED1,LED2,LED3,LED4);

#define NUMBER_FILE_DIRECTORY "/local/"
#define FILE_NAME_TEMPLATE "test"
#define FILE_NAME_EXTENTION "csv"
static const char *const filename = NUMBER_FILE_DIRECTORY FILE_NAME_TEMPLATE"."FILE_NAME_EXTENTION;
#define NUM_OF_NUME (2)
int remove_flag = 0;
int number_flag = 0;
int numidx = 0;
int numbers[NUM_OF_NUME];


void onKey(uint8_t);

int mode_number(int);
int mode_show(int);
void switch_mode(mode_key_t *);
void setNum(int);//'0'~'9' is guaranteed
file_write_t isExisting();
void deleteNum();
void prenoteNum();
FILE *openFile(const char *mode);
void removeFile();
void showNum();
void showList();
int saveNum();


void onKey(uint8_t key)
{
    static mode_key_t state_mode = Number;
    switch (key) {
        case 's':
            switch_mode(&state_mode);
            printf("mode change:%s\r\n",mode_string[state_mode]);
            break;
    }
    if((Number == state_mode ? mode_number : mode_show)(key)) {
        switch_mode(&state_mode);
        printf("auto change:%s\r\n",mode_string[state_mode]);
    }
}

inline void switch_mode(mode_key_t *mode)
{
    *mode = Number == *mode ? Show : Number;
}

int mode_number(int key)
{
    static num_state_t state = Init;
    switch (state) {
        case    Prenote:
            if ('\r'==key || '\n'==key) {
                showNum();
                saveNum();
                deleteNum();
            }
            state = Wait;
            if (0) {
            case Init:
                deleteNum();
                state = Wait;
            }
        case Wait:
            switch (key) {
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                    setNum(key);
                    break;
                case '\n':
                case '\r'://this does not work after deleting.
                    if (number_flag) {
                        file_write_t file_state = isExisting();
                        if (NONE == file_state) {
                            prenoteNum();
                            state = Prenote;
                        } else if (EXISTING == file_state) {
                            printf("already exiting!\r\n");
                            deleteNum();
                        }
                    }
                    break;
            }
            break;
        default:
            break;
    }
    led = state;
    if (remove_flag) {
        remove_flag = 0;
        if ('y' == key)
            removeFile();
        else
            printf("Remove canceled...\r\n");
    }

    if ('d' == key) {
        printf("Do you want to remove File? [y/n] \"%s\"\r\n", filename);
        remove_flag = 1;
    } else if ('\b' == key) {
        deleteNum();
        state = Wait;
    }
    return 0;
}

int mode_show(int key)
{
//このままだとshow中にモード切り替えされるとseekは保持される。
    int c;
    static off_t seek=0;
    FILE *fp = openFile("r");
    if ('\r'==key || '\n'==key) {
        fseek(fp, seek, SEEK_SET);
        if (EOF != (fscanf(fp, "%d",&c))) {
            seek =ftell(fp);
            printf("%2d\r\n",c);
        } else {
            seek = 0;
            printf("EOF\r\n");
            fclose(fp);
            return 1;
        }
    }
    fclose(fp);
    return 0;
}

void debug_show(const char *str)
{
    if (numbers[numidx]) {
        printf("%9s :'",str);
        putchar(numbers[numidx]);
        putchar(numbers[1-numidx]);
        printf("'\r\n");
    }
}

void setNum(int ascii)
{
    if(!number_flag) {
        number_flag = 1;
    } else {
        printf("\e[1A\e[2K\e[1G");
    }
    numbers[numidx] = ascii;
    numidx = 1-numidx;
    debug_show("set");
}
void deleteNum()
{
    debug_show("Delete");
    numidx = 0;
    number_flag = 0;
    numbers[0]='0';
    numbers[1]='0';
}

void prenoteNum()
{
    debug_show("prenote");
}

void showNum()
{
    debug_show("show");
}

FILE *openFile(const char *mode)
{
    int file_existing = 0;
    FILE *fp;

    if (!file_existing) {
        if (NULL != (fp = fopen(filename, "r"))) {
            file_existing = 1;
            fclose(fp);
        } else {
            if (NULL != (fp =fopen(filename, "w"))) {
                file_existing = 1;
                printf("create file \"%s\"\r\n",filename);
                fclose(fp);
            } else {
                printf("unknow error \"%s\"\r\n",filename);
            }
        }
    }

    fp = fopen(filename, mode);
    return fp;
}

void removeFile()
{
    if(0 == remove(filename)) {
        printf("Removed\"%s\"\r\n",filename);
    } else {
        printf("could not remove file!\r\n");
    }
}

//存在の確認(retval:NONE or EXISTING)
file_write_t isExisting()
{
    FILE *fp = openFile("r");
    int c;
    char buff[2];
    int digit = 0;
    file_write_t ret = NONE;
    while (EOF != (c=getc(fp))) {
        if ('\r'==c || '\n'==c) {
            if (digit) {
                digit = 0;
                if (numbers[numidx] == buff[0] && numbers[1-numidx]==buff[1]) {
                    ret = EXISTING;
                    break;
                }
            }
        } else {
            buff[digit++] = c;
        }
    }
    fclose(fp);
    return ret;
}

//存在しなければ保存
int saveNum()
{
    file_write_t state = isExisting();
    switch (state) {
        case NONE: {
            FILE *fp = openFile("a");
            putc(numbers[numidx], fp);
            putc(numbers[1-numidx], fp);
            putc('\r', fp);
            putc('\n', fp);
            fclose(fp);
            debug_show("saved");
            return DONE;
        }
        case EXISTING:
            printf("number is existing\r\n");
            return CANTWRITE;
        default:
            break;
    }
    //  printf("WARN:not implimented %s", __func__);
    return UNKNOWN;
}


void keyboard_task(void const *)
{
    USBHostKeyboard keyboard;
    while(1) {
        while(!keyboard.connect()) {
            Thread::wait(500);
        }
        keyboard.attach(onKey);
        printf("[connected]\r\n");
        while(keyboard.connected()) {
            Thread::wait(500);
        }
    }
}
int main()
{
    Thread keyboardTask(keyboard_task, NULL, osPriorityNormal, 256 * 4);
    while(1) {
        Thread::wait(500);
    }
}
